@@ -1,10 +1,10 @@
 package Net::DNS::RR::TSIG;
 
 #
-# $Id: TSIG.pm 1042 2012-11-01 21:03:37Z willem $
+# $Id: TSIG.pm 1090 2012-12-21 22:11:31Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1042 $)[1];
+$VERSION = (qw$LastChangedRevision: 1090 $)[1];
 
 use base Net::DNS::RR;
 
@@ -27,14 +27,14 @@ use constant TSIG => typebyname qw(TSIG);
 use MIME::Base64;
 
 use constant TSIG_DEFAULT_ALGORITHM => 'HMAC-MD5.SIG-ALG.REG.INT';
-use constant TSIG_DEFAULT_FUDGE     => 300;
+use constant TSIG_DEFAULT_FUDGE	    => 300;
 
 
 sub decode_rdata {			## decode rdata from wire-format octet string
 	my $self = shift;
-	my ( $data, $offset ) = @_;
+	my ( $data, $offset, @opaque ) = @_;
 
-	( $self->{algorithm}, $offset ) = decode Net::DNS::DomainName(@_);
+	( $self->{algorithm}, $offset ) = decode Net::DNS::DomainName2535(@_);
 
 	# Design decision: Use 32 bits, which will work until the end of time()!
 	@{$self}{qw(time_signed fudge)} = unpack "\@$offset xxN n", $$data;
@@ -54,13 +54,18 @@ sub decode_rdata {			## decode rdata from wire-format octet string
 
 sub encode_rdata {			## encode rdata as wire-format octet string
 	my $self = shift;
+	my ( $offset, @opaque ) = @_;
 
-	my ( $offset, $hash, $packet ) = @_;
+	my ( $hash, $packet ) = @opaque;
+
+	my @additional = grep { $_->type ne 'TSIG' } @{$packet->{additional}};
+	$packet->{additional} = \@additional;
+
+	$self->class('ANY');
 
 	my $macbin = $self->macbin;
 	unless ($macbin) {
 		my $key	     = $self->key || return '';
-		my $sig_time = $self->time_signed;
 		my $function = $self->sign_func;
 		my $sigdata  = $self->sig_data($packet);	# form data to be signed
 		$macbin = $self->macbin( &$function( $key, $sigdata ) );
@@ -78,6 +83,8 @@ sub encode_rdata {			## encode rdata as wire-format octet string
 
 	my $other = $self->other || '';
 	$rdata .= pack 'na*', length($other), $other;
+
+	push @{$packet->{additional}}, $self;
 	return $rdata;
 }
 
@@ -85,14 +92,25 @@ sub encode_rdata {			## encode rdata as wire-format octet string
 sub format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	join ' ', $self->algorithm, $self->error, $self->other || '';
+	my $fixed = join( ' ', ';', $self->algorithm, $self->error, $self->other || '' );
+	my @other = (
+		join( ' ', '; time signed', $self->time_signed, 'fudge', $self->fudge ),
+		join( ' ', '; original id', $self->original_id ) );
+	join "\n", $fixed, @other;
 }
 
 
 sub parse_rdata {			## populate RR from rdata in argument list
 	my $self = shift;
 
-	$self->key(@_) if @_;
+	$self->key(@_);
+}
+
+
+sub defaults() {			## specify RR attribute default values
+	my $self = shift;
+
+	$self->algorithm(TSIG_DEFAULT_ALGORITHM);
 }
 
 
@@ -125,29 +143,28 @@ sub encode {				## overide RR method
 sub algorithm {
 	my $self = shift;
 
-	$self->{algorithm} = new Net::DNS::DomainName(shift) if @_;
-	$self->{algorithm} ||= new Net::DNS::DomainName(TSIG_DEFAULT_ALGORITHM);
+	$self->{algorithm} = new Net::DNS::DomainName2535(shift) if scalar @_;
 	$self->{algorithm}->name if defined wantarray;
 }
 
 sub key {
 	my $self = shift;
 
-	$self->{key} = shift if @_;
+	$self->{key} = join '', @_ if scalar @_;
 	$self->{key} || "";
 }
 
 sub time_signed {
 	my $self = shift;
 
-	$self->{time_signed} = shift if @_;
+	$self->{time_signed} = shift if scalar @_;
 	return 0 + ( $self->{time_signed} || time() );
 }
 
 sub fudge {
 	my $self = shift;
 
-	$self->{fudge} = shift if @_;
+	$self->{fudge} = shift if scalar @_;
 	return 0 + ( $self->{fudge} || TSIG_DEFAULT_FUDGE );
 }
 
@@ -158,27 +175,41 @@ sub mac {
 sub macbin {
 	my $self = shift;
 
-	$self->{macbin} = shift if @_;
+	$self->{macbin} = shift if scalar @_;
 	$self->{macbin} || "";
 }
 
+sub request_mac {
+	my $self = shift;
+
+	$self->{request_mac} = shift if scalar @_;
+	$self->{request_mac} || '';
+}
+
+sub continuation {
+	my $self = shift;
+
+	$self->{continuation} = shift if scalar @_;
+	$self->{continuation} || 0;
+}
+
 sub original_id {
 	my $self = shift;
 
-	$self->{original_id} = shift if @_;
+	$self->{original_id} = shift if scalar @_;
 	return 0 + ( $self->{original_id} || 0 );
 }
 
 sub error {
 	my $self = shift;
-	$self->{error} = rcodebyname(shift) if @_;
+	$self->{error} = rcodebyname(shift) if scalar @_;
 	rcodebyval( $self->{error} || 0 );
 }
 
 sub other {
 	my $self = shift;
 
-	$self->{other} = shift if @_;
+	$self->{other} = shift if scalar @_;
 	return 0 + ( $self->{other} || 0 );
 }
 
@@ -187,30 +218,30 @@ sub other_data {&other}
 sub sign_func {
 	my $self = shift;
 
-	$self->{sign_func} = shift if @_;
+	$self->{sign_func} = shift if scalar @_;
 	$self->{sign_func} || \&_sign_hmac;
 }
 
 sub sig_data {
 	my ( $self, $packet ) = @_;
 
-	my @additional = grep { $_->type ne 'TSIG' } @{$packet->{additional}};
-	$packet->{additional} = [@additional];
-
 	# Add the request MAC if present (used to validate responses).
 	my $sigdata = '';
 	$sigdata = pack 'H*', $self->{request_mac} if $self->{request_mac};
 
 	$sigdata .= $packet->data;
-	push @{$packet->{additional}}, $self;
-
-	my $kname = $self->{owner}->encode(0);			# uncompressed key name
-	$sigdata .= pack 'a* n N', $kname, ANY, 0;
-
-	$sigdata .= $self->{algorithm}->encode();		# uncompressed algorithm name
 
 	# Design decision: Use 32 bits, which will work until the end of time()!
-	$sigdata .= pack 'xxN n', $self->{time_signed}, $self->fudge;
+	my $time = pack 'xxN n', $self->time_signed, $self->fudge;
+
+	return $sigdata . $time if $self->continuation;
+
+	my $kname = new Net::DNS::DomainName2535($self->{owner}->name );
+	$sigdata .= pack 'a* n N', $kname->encode(0), ANY, 0;	# canonical key name
+
+	$sigdata .= $self->{algorithm}->encode(0);		# canonical algorithm name
+
+	$sigdata .= $time;
 
 	$sigdata .= pack 'n', $self->{error} || 0;
 
@@ -252,7 +283,7 @@ A domain name which specifies the name of the algorithm.
 
     $key = $rr->key;
 
-Base64 encoded key.
+Key string in the format expected by the signing function.
 
 =head2 time_signed
 
@@ -283,6 +314,20 @@ object method before this will return anything meaningful.
 
 Binary message authentication code (MAC).
 
+=head2 request_mac
+
+     $tsig->request_mac( $request->macbin );
+
+Request message authentication code (MAC).
+
+
+=head2 continuation
+
+     $tsig->continuation(1);
+
+Flag which indicates continuation of a multi-message response.
+
+
 =head2 original_id
 
     $original_id = $rr->original_id;
